package gameplay;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Scanner;

public class EnemyCreator {
    private EnemyManager enemymanager;
    private double createrate, createtime, totaltime, missilecounter;
    private Enemy[] enemylist;
    private boolean missileTime;

    public EnemyCreator(EnemyManager enemymanager, double createrate) throws FileNotFoundException, IOException {
        this.enemymanager = enemymanager;

        this.createrate = 1.0/createrate;
        this.createtime = this.createrate;

        enemyInitialize("data/gameplay/enemies.txt");

        missileTime = false;
    }

    private void enemyInitialize(String filepath) throws FileNotFoundException, IOException {
        FileReader file = new FileReader(filepath);
        Scanner reader = new Scanner(file);

        enemylist = new Enemy[reader.nextInt()];

        for(int i = 0; i < enemylist.length; i++) {
            enemylist[i] = new Enemy(enemymanager.limitx, 0.0, reader.next(), reader.nextInt(), AI.getNivelOf(reader.nextInt()), reader.nextInt());
            enemylist[i].setFeatures(reader.nextDouble(), reader.nextDouble(), reader.nextDouble(), reader.nextDouble());
            enemylist[i].setBulletType(reader.nextDouble(), reader.nextDouble(), reader.next());
        }

        file.close();
    }

    public void Action(double elapsed) {
        totaltime += elapsed;

        createtime += elapsed;

        if(!missileTime) {
            if(createtime >= createrate/(1.0 + totaltime/150.0) || enemymanager.shipsInList() <= 0) {
                Create();

                createtime = 0;
            }
        } else {
            if(createtime >= 0.5) {
                createMissile();

                createtime = 0;
            }

            if(totaltime - missilecounter > 10.0)
                missileTime = false;
        }
    }

    private void Create() {
        if(totaltime < 45.0)
            createEasy();
        else if(totaltime < 90.0)
            createMedium();
        else if(totaltime < 135.0)
            createHard();
        else
            createExpert();
    }

    private void createEasy() {
        int j = (int)Math.round(Math.random()*3.0);

        enemymanager.Create(enemymanager.limitx, Math.random() * (enemymanager.limity - enemylist[j].height),
                            enemylist[j].image, enemylist[j].frames, enemylist[j].ai.nivel, enemylist[j].ID,
                            enemylist[j].bulletdamage, enemylist[j].bulletveloc, enemylist[j].bulletimg,
                            enemylist[j].agility, enemylist[j].maxspeed, 1000.0/enemylist[j].firerate, enemylist[j].casco);
    }

    private void createMedium() {
        int j = (int)Math.round(Math.random()*4.0 + 2.0);

        for(int i = (int)(Math.round(Math.random()*2) + 1.0); i > 0; i--) {
            enemymanager.Create(enemymanager.limitx, Math.random() * (enemymanager.limity - enemylist[j].height),
                            enemylist[j].image, enemylist[j].frames, enemylist[j].ai.nivel, enemylist[j].ID,
                            enemylist[j].bulletdamage + totaltime/60.0, enemylist[j].bulletveloc + totaltime/3.0, enemylist[j].bulletimg,
                            enemylist[j].agility + totaltime/6.0, enemylist[j].maxspeed + totaltime/6.0,
                            1000.0/enemylist[j].firerate + totaltime/120.0, enemylist[j].casco + totaltime/12.0);
        }
    }

    private void createHard() {
        int j = (int)Math.round(Math.random()*5.0 + 5.0);

        j = j >= 2 && j < 5 ? 2 : j;

        if(j > 9) {
            missileTime = true;

            missilecounter = totaltime;
        }  else {
            for(int i = (int)(Math.round(Math.random()*2) + 1.0); i > 0; i--) {
                enemymanager.Create(enemymanager.limitx, Math.random() * (enemymanager.limity - enemylist[j].height),
                            enemylist[j].image, enemylist[j].frames, enemylist[j].ai.nivel, enemylist[j].ID,
                            enemylist[j].bulletdamage + totaltime/60.0, enemylist[j].bulletveloc + totaltime/3.0, enemylist[j].bulletimg,
                            enemylist[j].agility + totaltime/6.0, enemylist[j].maxspeed + totaltime/6.0,
                            1000.0/enemylist[j].firerate + totaltime/120.0, enemylist[j].casco + totaltime/12.0);
            }
        }
    }

    private void createExpert() {
        int j = (int)Math.round(Math.random()*9.0 + 7.0);

        if(j > 15) {
            missileTime = true;

            missilecounter = totaltime;
        }  else {
            for(int i = (int)(Math.round(Math.random()*1.0) + 1.0); i > 0; i--) {
                enemymanager.Create(enemymanager.limitx, Math.random() * (enemymanager.limity - enemylist[j].height),
                            enemylist[j].image, enemylist[j].frames, enemylist[j].ai.nivel, enemylist[j].ID,
                            enemylist[j].bulletdamage + totaltime/30.0, enemylist[j].bulletveloc + totaltime/3.0, enemylist[j].bulletimg,
                            enemylist[j].agility + totaltime/3.0, enemylist[j].maxspeed + totaltime/6.0,
                            1000.0/enemylist[j].firerate + totaltime/60.0, enemylist[j].casco + totaltime/6.0);
            }
        }
    }

    private void createMissile() {
        int j = enemylist.length - 1;

        enemymanager.Create(enemymanager.limitx, Math.random() * (enemymanager.limity - enemylist[j].height),
                            enemylist[j].image, enemylist[j].frames, enemylist[j].ai.nivel, enemylist[j].ID,
                            enemylist[j].bulletdamage + totaltime/60.0, enemylist[j].bulletveloc + totaltime/3.0, enemylist[j].bulletimg,
                            enemylist[j].agility + totaltime/6.0, enemylist[j].maxspeed + totaltime/6.0,
                            1000.0/enemylist[j].firerate + totaltime/120.0, enemylist[j].casco + totaltime/12.0);
    }

    public double getTotalTime() {
        return totaltime;
    }
}
