package gameplay;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.awt.event.KeyEvent;
import JPlay.*;
import gameplay.hud.HUD;
import ranking.*;
import java.awt.Color;
import java.util.logging.Level;
import java.util.logging.Logger;
import shop.*;
import soundpack.*;

public class Space {
    private ArrayList<Enemy>    enemylist = new ArrayList();
    private Player              player;
    private BulletManager       bulletmanager;
    private EnemyManager        enemymanager;
    private EnemyCreator        enemycreator;
    private Background          background;
    private HUD                 hud;
    private Shop                shop;
    private SoundManager        soundmanager;

    private Window      app;
    private Keyboard    input;

    public Space(Window app) throws FileNotFoundException, IOException {
        this.app = app;

        input = this.app.getKeyboard();
        input.addKey(KeyEvent.VK_A, Keyboard.DETECT_EVERY_PRESS);
        input.addKey(KeyEvent.VK_W, Keyboard.DETECT_EVERY_PRESS);
        input.addKey(KeyEvent.VK_S, Keyboard.DETECT_EVERY_PRESS);
        input.addKey(KeyEvent.VK_D, Keyboard.DETECT_EVERY_PRESS);
        input.addKey(KeyEvent.VK_E, Keyboard.DETECT_INITIAL_PRESS_ONLY);
        input.addKey(KeyEvent.VK_Q, Keyboard.DETECT_INITIAL_PRESS_ONLY);

        player = new Player(10, 10, "data/gameplay/player.png", 13, app);
        player.setBulletType(5.0, 400.0, "data/gameplay/bullet5.png");

        Enemy.setAim(player);

        bulletmanager = new BulletManager(app);

        enemymanager = new EnemyManager(app);

        enemycreator = new EnemyCreator(enemymanager, 0.1);

        background = new Background(app, "data/gameplay/particle.png", "data/gameplay/deep.png", 60.0, 1800.0);

        hud = new HUD(app, player);

        shop = new Shop(app, "data/shop/background.png");

        soundmanager = new SoundManager();
        soundmanager.loadMusic("data/music/music1.wav", 60.0);
        soundmanager.loadMusic("data/music/music2.wav", 60.0);
        soundmanager.loadMusic("data/music/music3.wav", 60.0);
        soundmanager.loadMusic("data/music/music4.wav", 60.0);
        soundmanager.loadMusic("data/music/music5.wav", 60.0);
        soundmanager.loadMusic("data/music/music6.wav", 60.0);
        soundmanager.loadMusic("data/music/music7.wav", 60.0);
        soundmanager.loadMusic("data/music/music8.wav", 60.0);
        soundmanager.loadMusic("data/music/music9.wav", 60.0);
        soundmanager.loadMusic("data/music/music10.wav", 60.0);
        soundmanager.loadMusic("data/music/music11.wav", 60.0);

        soundmanager.loadSound("data/sounds/shot1.wav");
        soundmanager.loadSound("data/sounds/shot2.wav");

        Spaceship.setBulletManager(bulletmanager);
        Spaceship.setSoundManager(soundmanager);
    }

    public void Run() throws FileNotFoundException, IOException {
        player.setFeatures(500.0, 200.0, 5.0, 300.0);

        soundmanager.playList((int)Math.round(Math.random()*10.0));

        gameLoop();

        saveRecord();

        soundmanager.Stop();
        soundmanager.Close();
    }

    private void gameLoop() {
        boolean running = true;

        app.clear();
        Tutorial tuto = new Tutorial(app, 200, 100);
        tuto.addLine("Aqui estão algumas breves intruções sobre o jogo");
        tuto.addLine("Para ativar/desativar a arma da nave, dê um click com o botão esquerdo do mouse");
        tuto.addLine("Para abrir a loja de itens, dê um click com o botão direito do mouse");
        tuto.addLine("Para mover a nave, use as teclas WASD ou as setas do teclado");
        tuto.Run();

        while(running) {
            if(input.keyDown(Keyboard.ESCAPE_KEY)) {
                running = false;

                continue;
            }

            enemycreator.Action((double)app.timeElapsed()/1000.0);

            bulletmanager.Action(app.timeElapsed());

            enemymanager.Action(app.timeElapsed());

            background.Action((double)app.timeElapsed()/1000.0);

            player.Action(app.timeElapsed());

            soundmanager.Action((double)app.timeElapsed()/1000.0);

            player.Collision(enemymanager.checkCollision(player));

            player.money += enemymanager.getMoney();

            hud.Refresh();

            if(app.getMouse().isRightButtonPressed())
                shop.Open(player);

            if(app.getKeyboard().keyDown(KeyEvent.VK_E))
                soundmanager.nextSong();
            if(app.getKeyboard().keyDown(KeyEvent.VK_Q))
                soundmanager.previousSong();


            Draw();

            if(player.dead) {
                app.delay(1000);

                running = false;
            }
        }
    }

    public void Draw() {
        app.clear();
        background.Draw();
        enemymanager.Draw();
        player.draw();
        bulletmanager.Draw();
        hud.Draw();
        app.display();
    }

    private void saveRecord() {
        RankHandle rank = new RankHandle();
        String name = javax.swing.JOptionPane.showInputDialog("Digite seu nome");

        if(name != null) {
            if(!name.equals("")) {
                rank.loadRanking("data/ranking.txt");
                rank.Add(new Position(name, (int)enemycreator.getTotalTime()));
                rank.saveRanking("data/ranking.txt");
            }
        }
    }
}