package gameplay;

import JPlay.*;
import soundpack.*;

public class Spaceship extends Sprite {
    public double    casco, firerate;
    public double    velocity_x, velocity_y, agility, maxspeed;
    public double    bulletdamage, bulletveloc;
    public String    bulletimg;
    public boolean   alive, dead;
    public int       ID; // Identification
    protected int    frames, framecounter, anim_fps, anim_counter, firetime;

    public static BulletManager bulletmanager;
    protected static SoundManager soundmanager;

    public Spaceship(String img, int frames, int ID) { // The constructor
        super(img, frames);

        alive = true;
        dead = false;
        this.frames = frames;

        this.ID = ID;

        framecounter = 0;

        anim_fps = 45;
    }

    public void Action(long elapsed) { // This one calls all action methods
        if(alive) {
            this.Control((double)elapsed/1000.0);
            this.Move(elapsed);
            this.checkBulletCollision();
        } else
            dieAnimation(elapsed);
        this.checkHealth();
    }

    protected void Control(double elapsed) { // This method is gonna be override

    }

    private void checkHealth() { // Verifys if the spaceship has already "died"
        if(casco <= 0 && alive) {
            alive = false;
            
            new Sound("data/sounds/explosion1.wav").play();
        }

        if(framecounter >= frames)
            dead = true;
    }

    private void dieAnimation(long elapsed) { // Animation of the spaceship "death"
        anim_counter += elapsed;

        if(!dead && anim_counter >= anim_fps) {
            this.setCurrFrame(framecounter++);
            
            anim_counter = 0;
        }
    }

    public void setFeatures(double agility, double maxspeed, double firerate, double casco) { // Set the features of the spaceship
        this.agility = agility;
        this.maxspeed = maxspeed;
        this.firerate = 1000.0/firerate;
        this.casco = casco;
        this.firetime = (int)firerate;
    }

    public void setBulletType(double damage, double bulletveloc, String img) { // Set the kind of bullet is being used
        this.bulletdamage = damage;
        this.bulletveloc = bulletveloc;
        this.bulletimg = img;
    }

    protected void Move(long elapsed) { // Move the spaceship
        x += velocity_x*((double)elapsed/1000.0);
        y += velocity_y*((double)elapsed/1000.0);
    }

    protected void checkBulletCollision() { // Verifys if any bullet has collided with the spaceship
        casco -= bulletmanager.checkCollision(this);
    }

    public static void setBulletManager(BulletManager bulletm) { // Set the bullet manager the spaceship is using
        bulletmanager = bulletm;
    }

    public static void setSoundManager(SoundManager soundmanager) {
        Spaceship.soundmanager = soundmanager;
    }
}
