package gameplay;

import JPlay.*;
import java.awt.Color;
import java.util.ArrayList;
import java.util.Scanner;
import java.io.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import menupack.Button;

public class Tutorial {
    private Window              app;
    private ArrayList<String>   message;
    private Button              button1, button2;
    private boolean             state; // Verifica se o tutorial deverá ser exibido
    private int                 x, y;

    public Tutorial(Window app, int x, int y) {
        this.app = app;

        this.message = new ArrayList<String>();

        this.button1 = new Button("data/showagain.png", x, 0, app.getMouse());
        this.button2 = new Button("data/nevershow.png", x + button1.getSprite().width + 10, 0, app.getMouse());

        this.x = x;
        this.y = y;

        loadState();
    }

    public void addLine(String text) { // Adiciona uma linha à mensagem
        message.add(text);

        button1.getSprite().y = button2.getSprite().y = y + message.size()*20; // Atualiza a posição dos botões
    }

    public void Run() { // Roda o tutorial
        if(state) // Se ele estiver setado para rodar
            Loop();
    }

    private void Loop() { // Loop lógico do tutorial
        while(true) {
            Draw();

            if(button1.isMouseOn() && app.getMouse().isLeftButtonPressed()) // Se botão 1 for pressionado, sai do tuto
                break;
            if(button2.isMouseOn() && app.getMouse().isLeftButtonPressed()) { // Se botão 2, salva o estado para não mais abrir e sai
                state = false;

                saveState();
                
                break;
            }
        }
    }

    private void Draw() { // Desenha o tuto
        for(int i = 0; i < message.size(); i++) // desenha cada linha
            app.drawText(message.get(i), x, y + i*20, Color.yellow);

        button1.draw();
        button2.draw();

        app.display();
    }

    private void loadState() { // Carrega o estado do tuto a partir de um arquivo
        try {
            FileReader file = new FileReader("data/tutorialstate.txt");
            Scanner reader = new Scanner(file);

            state = reader.nextBoolean();

            reader.close();

            file.close();
        } catch (FileNotFoundException ex) { // Se o arquivo não existir, cria um
            try {
                FileWriter file = new FileWriter("data/tutorialstate.txt");

                file.write("" + true);

                state = true;

                file.close();
            } catch (IOException ex1) {
                Logger.getLogger(Tutorial.class.getName()).log(Level.SEVERE, null, ex1);
            }
        } catch (IOException ex) {
            Logger.getLogger(Tutorial.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    private void saveState() { // Salva o estado num arquivo
        FileWriter file;
        try {
            file = new FileWriter("data/tutorialstate.txt");

            file.write("" + state);

            file.close();
        } catch (IOException ex) {
            Logger.getLogger(Tutorial.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
}
