package ranking;

import JPlay.*;
import menupack.*;
import soundpack.*;

public class Ranking {
    private Window      app;
    private Keyboard    input;
    private Mouse       mouse;
    private GameImage   background;
    private RankList    list;
    private Button      up, down;
    private Music       music;

    public Ranking(Window app) {
        this.app = app;
        this.input = app.getKeyboard();
        this.mouse = app.getMouse();

        this.list = new RankList(app, 20, 25, 300);
        this.list.setPosition(app.getWidth()/2 - 150, 100);

        this.up = new Button("data/ranking/up.png", app.getWidth()/2 - 75, 20, app.getMouse());
        this.down = new Button("data/ranking/down.png", app.getWidth()/2 - 75, 510, app.getMouse());

        this.music = new Music("data/music/music6.wav", 60.0, 0);
    }

    public void Run() {
        fillUpList();

        music.Start();

        Loop();

        music.Stop();
        music.Close();
    }

    private void fillUpList() { // Abre um arquivo de ranking e preenche a lista
        RankHandle rank = new RankHandle();
        rank.loadRanking("data/ranking.txt");

        for(int i = 0; i < rank.Size(); i++)
            list.Add(rank.getRecord(i));
    }

    private void Loop() { // Loop do ranking
        while(!input.keyDown(Keyboard.ESCAPE_KEY)) { // Quando apertar esc sai
            if(mouse.isLeftButtonPressed()) { // verifica se o botão esquerdo foi clickado
                if(up.isMouseOn()) // Se clickou no botão up, então move a lista pra cima
                    list.moveUp();
                else if(down.isMouseOn()) // Se clickou no down, move pra baixo
                    list.moveDown();
            }

            Draw();
        }
    }

    private void Draw() { // Desenha o ranking
        if(background != null)
            background.draw();
        else
            app.clear(java.awt.Color.WHITE);

        list.Draw();
        up.draw();
        down.draw();

        app.display();
    }

    public void setBackground(String url) {
        background = new GameImage(url);
    }
}
