package soundpack;

import JPlay.*;
import java.io.*;
import java.util.Scanner;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.sound.sampled.*;

public class Music {
    private Clip   music;
    private boolean isover; // Diz se a música acabou
    private double  fadetime, totaltime, // Tempo do efeito em segundos e o tempo total da música
                    time; // Tempo da música
    private int     repeat; // quantas vezes a música vai repetir depois do início
    private float   volume; // Volume do som

    public Music(String music, double time) { // Construtor
        this(music, time, 1);
    }

    public Music(String music, double time, int repeat) {
        loadMusic(music);

        this.time = time;
        this.repeat = repeat;

        this.isover = false;

        this.volume = 0.0f;
    }

    public void loadMusic(String music) {
        File soundFile = new File(music);

        try {
            AudioInputStream audioIn = AudioSystem.getAudioInputStream(soundFile);

            this.music = AudioSystem.getClip();

            this.music.open(audioIn);
        } catch (UnsupportedAudioFileException ex) {
            Logger.getLogger(SoundManager.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(SoundManager.class.getName()).log(Level.SEVERE, null, ex);
        } catch (LineUnavailableException ex) {
            Logger.getLogger(SoundManager.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void Start() { // Inicia a música
        totaltime = 0.0;
        isover = false;

        music.loop(music.LOOP_CONTINUOUSLY);
    }

    public void Stop() { // Para a música
        music.stop();
        music.setFramePosition(0);

        totaltime = 0.0;
        isover = true;
    }

    public void Action(double elapsed) {
        totaltime += elapsed;

        if(totaltime >= time * (double)repeat && repeat > 0)
            Stop();
    }

    public boolean isOver() { // Retorna true se a música acabou
        return isover;
    }

    public void Close() {
        music.close();
    }
}
