package soundpack;

import JPlay.*;
import java.util.ArrayList;
import java.io.*;
import java.util.Scanner;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.sound.sampled.*;

public class SoundManager {
    private ArrayList<Music> musiclist; // Lista de músicas
    private ArrayList<Clip> soundlist; // Lista de sons
    private boolean          playinglist; // Se for true, tocará uma música após a outra
    private int              index; // Posição da música na lista que está tocando

    public SoundManager() { // Construtor
        this.playinglist = false;
        this.index = 0;

        musiclist = new ArrayList<Music>();
        soundlist = new ArrayList<Clip>();
    }

    public void loadMusicList(String list) { // Carrega uma lista de músicas a partir de um arquivo, apaga a lista antiga
        try {
            FileReader file = new FileReader(list);
            Scanner reader = new Scanner(file);
            
            for(int i = reader.nextInt() - 1; i >= 0; i--)
                musiclist.add(new Music(reader.next(), reader.nextDouble(), reader.nextInt()));
          
            file.close();
        } catch (FileNotFoundException ex) {
            Logger.getLogger(SoundManager.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(SoundManager.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void loadMusic(String music, double time) { // Adiciona uma música ao final da lista atual
        musiclist.add(new Music(music, time));
    }

    public void loadSoundList(String list) { // Carrega uma lista de sons, apaga a lista antiga
        try {
            FileReader file = new FileReader(list);
            Scanner reader = new Scanner(file);

            soundlist.clear();
            for(int i = reader.nextInt() - 1; i >= 0; i--)
                loadSound(reader.next());

            file.close();
        } catch (FileNotFoundException ex) {
            Logger.getLogger(SoundManager.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(SoundManager.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void loadSound(String sound) { // Adiciona um son ao final da lista
        File soundFile = new File(sound);

        try {
            AudioInputStream audioIn = AudioSystem.getAudioInputStream(soundFile);

            Clip clip = AudioSystem.getClip();

            clip.open(audioIn);

            soundlist.add(clip);
        } catch (UnsupportedAudioFileException ex) {
            Logger.getLogger(SoundManager.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(SoundManager.class.getName()).log(Level.SEVERE, null, ex);
        } catch (LineUnavailableException ex) {
            Logger.getLogger(SoundManager.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public void Stop() { // Para a música
        musiclist.get(index).Stop();
    }

    public void playMusic(int index) { // Toca uma única música da lista, se repeat for zero repete infinitas vezes
        if(index < musiclist.size()) {
            playinglist = false;

            musiclist.get(this.index).Stop();

            this.index = index;

            musiclist.get(this.index).Start();
        }
    }

    public void playList(int index) { // Toca a lista, uma música após a outra, começa pelo index e cada música vai durar timeofeachsong
        if(index < musiclist.size()) {
            playinglist = true;

            musiclist.get(this.index).Stop();

            this.index = index;

            musiclist.get(this.index).Start();
        }
    }

    public void playSound(int index) {
        if(index < soundlist.size()) {
            if(soundlist.get(index).isRunning())
                soundlist.get(index).stop();

            soundlist.get(index).setFramePosition(0);
            soundlist.get(index).start();
        }
    }

    public void Action(double elapsed) {
        if(!musiclist.isEmpty())
            musiclist.get(index).Action(elapsed);

        if(playinglist)
            if(musiclist.get(index).isOver())
                nextSong();
    }

    public void nextSong() {
        musiclist.get(index).Stop();

        index = ++index >= musiclist.size() ? 0 : index;

        musiclist.get(index).Start();
    }

    public void previousSong() {
        musiclist.get(index).Stop();

        index = --index < 0 ? musiclist.size() - 1 : index;

        musiclist.get(index).Start();
    }

    public int sizeOfMusicList() {
        return musiclist.size();
    }

    public void Close() {
        for(int i = 0; i < musiclist.size(); i++)
            musiclist.get(i).Close();

        for(int i = 0; i < soundlist.size(); i++)
            soundlist.get(i).close();
    }
}
